@@ -1,3 +1,4 @@
+
 import logging
 import platform
 import gspm.utils.path_utils as path_utils
@@ -5,9 +6,11 @@ import wget
 import zipfile
 import os
 import gspm.utils.process_utils as process_utils
-from packaging.version import Version
 import subprocess
 
+from packaging.version import Version
+from gspm.utils.versions import *
+
 
 host_url = "https://downloads.tuxfamily.org/godotengine/{0}/"
 
@@ -28,11 +31,11 @@ def install_godot(project):
     if project.config.godot.local:
         pass
     else:
-        if Version('{0}'.format(project.config.godot.version)) < Version('2.1'):
-            raise Exception("Version [{0}] Not Supported".format(project.config.godot.version))
+        # if Version('{0}'.format(project.config.godot.version)) < Version('2.1'):
+        #     raise Exception("Version [{0}] Not Supported".format(project.config.godot.version))
 
-        if Version('{0}'.format(project.config.godot.version)) > Version('3.2.3'):
-            raise Exception("Version [{0}] Not Supported".format(project.config.godot.version))
+        # if Version('{0}'.format(project.config.godot.version)) > Version('3.2.3'):
+        #     raise Exception("Version [{0}] Not Supported".format(project.config.godot.version))
 
         logging.debug(
             "- checking for godot [{0}]".format(project.config.godot.version))
@@ -56,11 +59,22 @@ def install_godot(project):
 
         uri = _build_godot_uri(project)
 
-        logging.debug(
-            "- downloading [{0}] to [{1}]".format(uri, dest_path))
+        _mono = ''
+        if (project.config.godot.mono):
+            _mono = ' for mono'
+
+        _version = "Version [{0}-{1}-{2}]{3} on [{4}]".format(project.config.godot.version, project.config.godot.release, project.config.godot.arch, _mono, _get_platform())
+
+        if (uri == '*none'):
+            raise Exception("Version {0} Not Supported.".format(_version))
+
+        project.config.uri = uri
 
         logging.log(99,
-            "getting godot [{0}]".format(project.config.godot.version))
+            "getting godot {0}".format(_version))
+
+        logging.debug(
+            "- downloading [{0}] to [{1}]".format(uri, dest_path))
 
         file = wget.download(uri, dest_path)
 
@@ -72,8 +86,7 @@ def install_godot(project):
         zipf.close()
 
         if _get_platform() == "darwin":
-            zfile = "{0}/Godot.app".format(dest_path)
-            subprocess.call(['chmod', '-R', '+x', zfile])
+            subprocess.call(['chmod', '-R', '+x', dest_path])
             
         if _get_platform() == "linux":
             subprocess.call(['chmod', '-R', '+x', dest_path])
@@ -107,6 +120,23 @@ def export_godot(project, name, path):
 
 def _build_godot_uri(project):
     logging.debug("[godot_utils] _build_godot_uri")
+    _uri = "*none"
+    _mono = ''
+    _system = _get_platform()
+
+    if (project.config.godot.mono == True):
+        _mono = 'mono'
+
+    _version = ':{0}:{1}:{2}:{3}:{4}'.format(project.config.godot.version, project.config.godot.release, _system, project.config.godot.arch, _mono )
+
+    if (_version in godot_versions):
+        _uri = godot_versions[_version]
+
+    return _uri
+
+
+def _build_godot_uriX(project):
+    logging.debug("[godot_utils] _build_godot_uri")
 
     system = _get_platform()
     uri = ""
@@ -262,10 +292,12 @@ def _build_darwin_cmd(project):
 
 #   return the platform we are running on
 def _get_platform():
+    logging.debug('[godot_utils] _get_platform')
     return platform.system().lower()
 
 
 def _get_godot_runtime(project):
+    logging.debug('[godot_utils] _get_godot_runtime')
     plat = _get_platform()
     runtime = ""
 
@@ -286,31 +318,36 @@ def _get_godot_runtime(project):
 
 def _get_windows_runtime(project):
     logging.debug('[godot_utils] _get_windows_runtime')
-    if Version('{0}'.format(project.config.godot.version)) < Version('3.3'):
-        stable = 'stable'
-    else:
-        stable = 'alpha3'
+    _uri = _build_godot_uri(project)
+    _runtime = _uri[_uri.rfind("/")+1:]
+    _runtime = _runtime.replace('.zip', '')
 
-    logging.debug(project)
-    runtime = "Godot_v{0}-{2}_win{1}.exe".format(project.config.godot.version, project.config.godot.arch, stable)
-    logging.debug(runtime)
-    return runtime
+    if (project.config.godot.mono):
+        _runtime = '{0}\{0}'.format(_runtime)
+        
+    return _runtime
 
 
 def _get_linux_runtime(project):
     logging.debug('[godot_utils] _get_linux_runtime')
-    if Version('{0}'.format(project.config.godot.version)) < Version('3.3'):
-        stable = 'stable'
-    else:
-        stable = 'alpha3'
+    _uri = _build_godot_uri(project)
+    _runtime = _uri[_uri.rfind("/")+1:]
+    _runtime = _runtime.replace('.zip', '')
 
-    runtime = "Godot_v{0}-{2}_x11.{1}".format(project.config.godot.version, project.config.godot.arch, stable)
-    #runtime = "Godot_v{0}-{2}_linux_headless.{1}".format(project.config.godot.version, project.config.godot.arch, stable)
-    return runtime
+    if (project.config.godot.mono):
+        _runtime = '{0}/{0}'.format(_runtime)
+        _s = list(_runtime)
+        _s[_runtime.rfind("_")] = '.'
+        _runtime = "".join(_s)
+
+    return _runtime
 
 
 def _get_darwin_runtime(project):
     logging.debug('[godot_utils] _get_darwin_runtime')
     runtime = "Godot.app/Contents/MacOS/Godot"
+    if (project.config.godot.mono):
+            runtime = "Godot_mono.app/Contents/MacOS/Godot"
+
     return runtime
 
